// ******************* BASIC.FRAG *******************

#version 330

struct Material {
	sampler2D diffuse;
	sampler2D specular;
	float shininess;
};

struct Light {
	vec3 position;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

in vec2 TexCoords;
in vec3 Normal;
in vec3 FragPos;
in vec4 FragPosLightSpace;

out vec4 color;

uniform vec3 viewPos;
uniform Material material;
uniform Light light;
uniform sampler2D shadowMap; //sampler2DShadow

float shadowCalculation(vec4 fragPosLightSpace)
{
	vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
	projCoords = projCoords * 0.5 + 0.5;
	float closestDepth = texture(shadowMap, projCoords.xy).r;
	float currentDepth = projCoords.z;
	//float shadow = currentDepth > closestDepth  ? 1.0 : 0.0;
	
	if (currentDepth > closestDepth)
		return 1.0;
	else
		return 0.0;
}

void main()
{
	vec3 tcolor = texture(material.diffuse, TexCoords).rgb;
	
	// Ambient Light
	vec3 ambient = light.ambient;
	
	// Diffuse Light
	vec3 norm = normalize(Normal);
	vec3 lightDir = normalize(light.position - FragPos);
	float diff = max(dot(norm, lightDir), 0.0);
	vec3 diffuse = light.diffuse * diff;

	// Specular Light
	// higher value means more concentrated highlight
	vec3 viewDir = normalize(viewPos - FragPos);
	vec3 reflectDir = reflect(-lightDir, norm);
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
	vec3 specular = light.specular * spec;
	
	float shadow = shadowCalculation(FragPosLightSpace);
	
	vec3 specularPart = specular * vec3(texture(material.specular, TexCoords));
	vec3 ambientPart = ambient * tcolor;
	vec3 diffusePart = diffuse * tcolor;
	
	vec3 result = ambientPart + (1.0 - shadow) * (diffusePart + specularPart);
	color = vec4(result, 1.0f);
}